"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runDecay = runDecay;
exports.runTiming = runTiming;
exports.runSpring = runSpring;
exports.normalizeSnapPoints = void 0;

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/** -------------------------------------------- */

/**          Utility functions                   */

/** -------------------------------------------- */
const magic = {
  damping: 50,
  mass: 0.3,
  stiffness: 121.6,
  overshootClamping: true,
  restSpeedThreshold: 0.3,
  restDisplacementThreshold: 0.3,
  deceleration: 0.999,
  bouncyFactor: 1,
  velocityFactor: 0.9,
  toss: 0.4,
  coefForTranslatingVelocities: 5
};
const {
  damping,
  mass,
  stiffness,
  overshootClamping,
  restSpeedThreshold,
  restDisplacementThreshold
} = magic;
const {
  set,
  cond,
  multiply,
  decay,
  lessThan,
  timing,
  block,
  min,
  max,
  Value,
  spring,
  clockRunning,
  startClock,
  stopClock,
  greaterOrEq
} = _reactNativeReanimated.default;
/**
 * Converts snap points with percentage to fixed numbers.
 */

const normalizeSnapPoints = (snapPoints, containerHeight) => snapPoints.map(snapPoint => {
  return typeof snapPoint === 'number' ? snapPoint : Number(snapPoint.split('%')[0]) * containerHeight / 100;
});

exports.normalizeSnapPoints = normalizeSnapPoints;

function runDecay(clock, value, velocity, positionToUpdate, wasStartedFromBegin) {
  const state = {
    finished: new Value(0),
    velocity: new Value(0),
    position: new Value(0),
    time: new Value(0)
  };
  const config = {
    deceleration: magic.deceleration
  };
  const resultScroll = max(min(state.position, -1), multiply(2992, -1));
  return block([cond(clockRunning(clock), 0, [set(state.finished, 0), set(state.velocity, multiply(velocity, magic.velocityFactor)), set(state.position, value), set(state.time, 0), startClock(clock)]), cond(greaterOrEq(resultScroll, 0), [set(state.finished, 1), stopClock(clock)]), cond(lessThan(resultScroll, multiply(2000, -1)), [set(state.finished, 1), stopClock(clock)]), cond(clockRunning(clock), [// positionToUpdate(state.position),
  set(positionToUpdate, resultScroll), decay(clock, state, config)]), cond(state.finished, [set(wasStartedFromBegin, 0), stopClock(clock)]), resultScroll]);
}

function runTiming(clock, value, dest, flagFinished) {
  const state = {
    finished: new Value(0),
    position: new Value(0),
    time: new Value(0),
    frameTime: new Value(0)
  };
  const config = {
    duration: 200,
    toValue: new Value(0),
    easing: _reactNativeReanimated.Easing.inOut(_reactNativeReanimated.Easing.ease)
  };
  return block([cond(clockRunning(clock), [// if the clock is already running we update the toValue, in case a new dest has been passed in
  set(config.toValue, dest)], [// if the clock isn't running we reset all the animation params and start the clock
  set(state.finished, 0), set(state.time, 0), set(state.position, value), set(state.frameTime, 0), set(config.toValue, dest), startClock(clock)]), // we run the step here that is going to update position
  timing(clock, state, config), // if the animation is over we stop the clock
  cond(state.finished, [stopClock(clock), set(flagFinished, 1)]), // we made the block return the updated position
  state.position]);
}

function runSpring(clock, start, end, velocity, updateWhenFinished, wasRun, valueToUpdate) {
  const state = {
    finished: new Value(0),
    velocity: new Value(0),
    position: new Value(0),
    time: new Value(0)
  };
  const config = {
    damping,
    mass,
    stiffness,
    overshootClamping,
    restSpeedThreshold,
    restDisplacementThreshold,
    toValue: new Value(0)
  };
  return block([cond(clockRunning(clock), [// if the clock is already running we update the toValue, in case a new dest has been passed in
  set(config.toValue, end)], [// if the clock isn't running we reset all the animation params and start the clock
  // set(forcedFlag, 0),
  set(state.finished, 0), set(state.position, start), set(state.velocity, velocity), set(state.time, 0), set(config.toValue, end), startClock(clock)]), spring(clock, state, config), cond(state.finished, [stopClock(clock), set(wasRun, 0), updateWhenFinished]), set(valueToUpdate, state.position), state.position]);
}
//# sourceMappingURL=utility.js.map