function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import * as React from 'react';
import { View, Dimensions, TouchableWithoutFeedback, StyleSheet } from 'react-native';
import { Navigation } from 'react-native-navigation';
import Animated from 'react-native-reanimated';
import { PanGestureHandler, TapGestureHandler, State as GestureState, gestureHandlerRootHOC } from 'react-native-gesture-handler';
import { listen, dispatch } from './events';
import { runSpring, normalizeSnapPoints } from './utility';
const {
  call,
  cond,
  greaterThan,
  lessThan,
  neq,
  clockRunning,
  not,
  and,
  set,
  sub,
  or,
  stopClock,
  lessOrEq,
  proc,
  add,
  max,
  min,
  eq,
  multiply,
  block,
  onChange
} = Animated;
const screenHeight = Dimensions.get('window').height;

class BottomSheet extends React.Component {
  /* BottomSheet state */

  /** -------------------------------------------- */

  /**          Declaration of variables            */

  /** -------------------------------------------- */

  /* Numerical values of snap points which are provided in props */

  /* Flag indicating that a command to close the bottom sheet was initiated. */

  /* Animated value responsible for an animation of the bottom sheet */

  /* Clock used for snapping animations of the bottom sheet */

  /* Clock used for an animation of a scrolling view (not implemented yet) */

  /* Snap points mapped to Animated values */

  /* Value of a current dragging postion */

  /* Value of a current speed of dragging */

  /* State of a gesture */

  /* Position of a last tap */

  /* Basically last snap point */

  /* A flag forcing an animation to run; variables below define the animation */

  /* Forcefully set starting point of a spring animation */

  /* Forcefully set destination point of a spring animation */

  /* Gesture mapping */

  /* Value of a current dragging postion of content in the scroll view */

  /* Value of a current speed of content dragging in the scroll view  */

  /* State of a gesture for a scroll view */

  /* Animated value mapped to _dragY from _scrollY */

  /* Dragging animation for bottom sheet, calculated as offset + dragging value */
  constructor(props) {
    super(props);

    _defineProperty(this, "state", {
      heightOfHeader: 0,
      contentHeight: new Animated.Value(0),
      screenHeight: screenHeight
    });

    _defineProperty(this, "snapPoints", normalizeSnapPoints(this.props.snapPoints, screenHeight));

    _defineProperty(this, "topSnap", this.snapPoints[this.snapPoints.length - 1]);

    _defineProperty(this, "closed", false);

    _defineProperty(this, "_masterTranslateY", void 0);

    _defineProperty(this, "_clock", new Animated.Clock());

    _defineProperty(this, "_scrollingClock", new Animated.Clock());

    _defineProperty(this, "_snapPoints", []);

    _defineProperty(this, "_dragY", new Animated.Value(0));

    _defineProperty(this, "_velocityY", new Animated.Value(0));

    _defineProperty(this, "_panState", new Animated.Value(GestureState.END));

    _defineProperty(this, "_absoluteY", new Animated.Value(0));

    _defineProperty(this, "_lastBottomSheetHeight", new Animated.Value(this.snapPoints[this.props.initialSnapIndex ? this.props.initialSnapIndex : this.snapPoints.length - 1]));

    _defineProperty(this, "_forcedSet", new Animated.Value(1));

    _defineProperty(this, "_startSnapPoint", new Animated.Value(screenHeight - this.snapPoints[0]));

    _defineProperty(this, "_endSnapPoint", new Animated.Value(screenHeight - this.snapPoints[this.props.initialSnapIndex ? this.props.initialSnapIndex : this.snapPoints.length - 1]));

    _defineProperty(this, "_onGestureEvent", Animated.event([{
      nativeEvent: {
        translationY: this._dragY,
        velocityY: this._velocityY,
        state: this._panState
      }
    }]));

    _defineProperty(this, "_scrollY", new Animated.Value(0));

    _defineProperty(this, "_lastScrollY", new Animated.Value(0));

    _defineProperty(this, "_masterScrollY", new Animated.Value(0));

    _defineProperty(this, "_velocityScrollY", new Animated.Value(0));

    _defineProperty(this, "_panScrollState", new Animated.Value(GestureState.END));

    _defineProperty(this, "_onGestureEventScrolling", Animated.event([{
      nativeEvent: {
        translationY: this._scrollY,
        velocityY: this._velocityScrollY,
        state: this._panScrollState
      }
    }]));

    _defineProperty(this, "_scrollToDragVal", new Animated.Value(0));

    _defineProperty(this, "_masterOpacity", void 0);

    _defineProperty(this, "_draggingAnimation", Animated.interpolate(Animated.add(Animated.sub(screenHeight, this._lastBottomSheetHeight), add(this._dragY, this._scrollToDragVal)), {
      inputRange: [screenHeight - this.snapPoints[this.snapPoints.length - 1], screenHeight - this.snapPoints[0]],
      outputRange: [screenHeight - this.snapPoints[this.snapPoints.length - 1], screenHeight - this.snapPoints[0]],
      extrapolate: Animated.Extrapolate.CLAMP
    }));

    _defineProperty(this, "handleTapEvent", ({
      event
    }) => {// if (!this.props.enabledContentGestureInteraction) {
      //   this._velocityScrollY.setValue(0);
      //   this._velocityY.setValue(0);
      // }
      // console.log('Tapped');
    });

    _defineProperty(this, "handleLayoutHeader", ({
      nativeEvent: {
        layout: {
          height: heightOfHeader
        }
      }
    }) => {
      this.setState({
        heightOfHeader
      });
    });

    _defineProperty(this, "handleLayoutContent", ({
      nativeEvent: {
        layout: {
          height
        }
      }
    }) => {
      const resultHeight = this.props.enabledContentGestureInteraction ? height + this.state.heightOfHeader - this.snapPoints[this.snapPoints.length - 1] : 0; // console.log(this.topSnap - this.state.heightOfHeader);

      this.state.contentHeight.setValue(Math.max(resultHeight, 0));
    });

    _defineProperty(this, "unsubscribeDismissBottomSheet", void 0);

    _defineProperty(this, "unsubscribeSnapTo", void 0);

    _defineProperty(this, "unsubscribeNavigationListener", void 0);

    _defineProperty(this, "unsubscribeNavigationBackButtonListener", void 0);

    _defineProperty(this, "registerListeners", () => {
      console.log('Registering listeners...');
      this.unsubscribeSnapTo = listen('BOTTOM_SHEET_SNAP_TO', index => {
        this.snapTo(screenHeight - this.snapPoints[index]);
      }); // Executes when the drawer needs to be dismissed

      this.unsubscribeDismissBottomSheet = listen('DISMISS_BOTTOM_SHEET', () => {
        this.closeBottomSheet();
      });
    });

    _defineProperty(this, "removeListeners", () => {
      if (this.unsubscribeNavigationListener) this.unsubscribeNavigationListener.remove();
      if (this.unsubscribeNavigationBackButtonListener) this.unsubscribeNavigationBackButtonListener.remove();
      if (this.unsubscribeSnapTo) this.unsubscribeSnapTo();
      if (this.unsubscribeDismissBottomSheet) this.unsubscribeDismissBottomSheet();
    });

    _defineProperty(this, "touchedOutside", () => {
      console.log('Touched outsied');
      const {
        dismissWhenTouchOutside
      } = this.props;

      if (dismissWhenTouchOutside) {
        this.closeBottomSheet();
      }
    });

    _defineProperty(this, "snapTo", endPoint => {
      this._endSnapPoint.setValue(endPoint);

      this._startSnapPoint.setValue(Animated.add(Animated.sub(screenHeight, this._lastBottomSheetHeight), this._dragY));

      this._forcedSet.setValue(1);
    });

    _defineProperty(this, "closeBottomSheet", () => {
      if (this.closed) return;
      console.log('Closing bottom sheet');
      this.closed = true;
      this.snapTo(screenHeight - this.snapPoints[0]);
      dispatch('MARK_CLOSED');
      setTimeout(() => Navigation.dismissModal(this.props.componentId), 250); // setTimeout(() => Navigation.dismissOverlay(this.props.componentId), 250);
    });

    for (let i = 0; i < this.snapPoints.length; i++) {
      this._snapPoints.push(new Animated.Value(this.snapPoints[i]));
    }

    const distance = Animated.proc((bottomPoint, upperPoint) => Animated.abs(Animated.sub(bottomPoint, upperPoint)));
    const currentPoint = Animated.sub(this._lastBottomSheetHeight, add(this._dragY, this._scrollToDragVal), Animated.multiply(0.25, this._velocityY));
    const rememberAndReturn = Animated.proc(value => Animated.block([Animated.set(this._lastBottomSheetHeight, value), value]));
    /**
     * Recursive function finding closest snap point.
     */

    const getClosestSnapPoint = (index = 0) => {
      return index === this._snapPoints.length - 1 ? rememberAndReturn(this._snapPoints[index]) : Animated.cond(Animated.greaterThan(distance(this._snapPoints[index + 1], currentPoint), distance(currentPoint, this._snapPoints[index])), rememberAndReturn(this._snapPoints[index]), getClosestSnapPoint(index + 1));
    }; // Does not let the bottom sheet to go higher the highest snap point
    // and below the lowest snap point


    const limitedAnimatedValue = value => {
      return Animated.cond(Animated.eq(this._lastBottomSheetHeight, this._snapPoints[0]), Animated.min(value, Animated.sub(screenHeight, this.snapPoints[0])), Animated.max(value, Animated.sub(screenHeight, this.snapPoints[this.snapPoints.length - 1])));
    };

    const generalDebug = (val, msg) => {
      console.log(msg);
      console.log(val[0]);
    };

    const snapPointUpdated = snapPoint => {
      if (this.props.onChange) {
        for (let i = 0; i < this.snapPoints.length; i++) if (this.snapPoints[i] === snapPoint[0]) {
          this.props.onChange(i);
          break;
        }
      }

      if (snapPoint[0] === 0) this.closeBottomSheet();
    };

    const curScroll = new Animated.Value(0);
    const resultScroll = max(min(add(this._lastScrollY, curScroll), 0), multiply(this.state.contentHeight, -1));
    const isRun = new Animated.Value(0);
    const storedResult = new Animated.Value(0);
    /**
     * Run a function finding a snap point only once. Save result in storedResult variable.
     */

    const runOnce = func => {
      return cond(isRun, storedResult, [set(storedResult, func), set(isRun, 1), set(this._dragY, 0), set(this._velocityY, 0), set(this._lastScrollY, resultScroll), set(this._scrollY, 0), set(this._scrollToDragVal, 0), set(this._velocityScrollY, 0), storedResult]);
    };

    const updateWhenFinished = block([cond(this._forcedSet, [set(this._lastBottomSheetHeight, sub(screenHeight, this._endSnapPoint)), set(this._forcedSet, 0)])]);
    const currentMovingPoint = new Animated.Value(0);
    /**
     * Stop the spring animation and reset some values.
     */

    const __stopClock = block([set(this._lastBottomSheetHeight, sub(screenHeight, currentMovingPoint)), updateWhenFinished, set(this._velocityScrollY, 0), set(this._velocityY, 0), set(isRun, 0), Animated.stopClock(this._clock)]);

    this._masterScrollY = block([cond(lessThan(sub(this._snapPoints[this._snapPoints.length - 1], this._lastBottomSheetHeight), 10), // if the bottom sheet is snapped to the very top
    [set(this._scrollToDragVal, max(add(this._scrollY, this._lastScrollY), 0)), set(curScroll, this._scrollY), cond(greaterThan(add(this._scrollY, this._lastScrollY), -1), set(this._velocityY, this._velocityScrollY))], [set(this._velocityY, this._velocityScrollY), set(this._scrollToDragVal, this._scrollY), set(curScroll, min(add(sub(this._snapPoints[this._snapPoints.length - 1], this._lastBottomSheetHeight), this._scrollY), 0))]), cond(eq(this._panScrollState, GestureState.BEGAN), set(this._velocityY, 0)), cond(or(Animated.eq(this._panScrollState, GestureState.END), Animated.eq(this._panScrollState, GestureState.CANCELLED), Animated.eq(this._panScrollState, GestureState.FAILED)), [set(this._lastScrollY, resultScroll)], [cond(clockRunning(this._clock), __stopClock)]), resultScroll]);
    const runSpringAnimation = new Animated.Value(0);
    this._masterTranslateY = Animated.block([onChange(this._lastBottomSheetHeight, call([this._lastBottomSheetHeight], snapPointUpdated)), // onChange(
    //   this._scrollToDragVal,
    //   call([this._scrollToDragVal], (val: any) => generalDebug(val ,"ScrollTodrag: ")),
    // ),
    set(runSpringAnimation, or(and(or(Animated.eq(this._panState, GestureState.END), Animated.eq(this._panState, GestureState.CANCELLED), Animated.eq(this._panState, GestureState.FAILED)), or(Animated.eq(this._panScrollState, GestureState.END), Animated.eq(this._panScrollState, GestureState.CANCELLED), Animated.eq(this._panScrollState, GestureState.FAILED)) // not(eq(add(this._lastBottomSheetHeight, this._dragY), 500)),
    ), and(greaterThan(this._forcedSet, 0), neq(this._panState, GestureState.ACTIVE)))), Animated.cond(runSpringAnimation, [runSpring(this._clock, limitedAnimatedValue(cond(this._forcedSet, this._startSnapPoint, add(sub(screenHeight, this._lastBottomSheetHeight), add(this._dragY, this._scrollToDragVal)))), cond(this._forcedSet, this._endSnapPoint, sub(screenHeight, runOnce(getClosestSnapPoint()))), this._velocityY, updateWhenFinished, isRun, currentMovingPoint)], [cond(Animated.clockRunning(this._clock), __stopClock), // Animated.stopClock(this.clock),
    this._draggingAnimation])]);
    this._masterOpacity = Animated.interpolate(this._masterTranslateY, {
      inputRange: [screenHeight - this.snapPoints[this.snapPoints.length - 1], screenHeight - this.snapPoints[0]],
      outputRange: [this.props.fadeOpacity, 0],
      extrapolate: Animated.Extrapolate.CLAMP
    });
    this.unsubscribeNavigationListener = Navigation.events().bindComponent(this);
    this.unsubscribeNavigationBackButtonListener = Navigation.events().registerNavigationButtonPressedListener(event => {
      if (event.buttonId === 'RNN.hardwareBackButton' || event.buttonId === 'RNN.hardwareBack') {
        this.closeBottomSheet();
      }
    });
    this.unsubscribeSnapTo = listen('BOTTOM_SHEET_SNAP_TO', index => {
      this.snapTo(screenHeight - this.snapPoints[index]);
    }); // Executes when the drawer needs to be dismissed

    this.unsubscribeDismissBottomSheet = listen('DISMISS_BOTTOM_SHEET', () => {
      this.closeBottomSheet();
    });
  }
  /** -------------------------------------------- */

  /**               Class methods                  */

  /** -------------------------------------------- */

  /**
   * [ react-native-navigation method. ]
   *
   * Executed when the component is navigated to view.
   */


  componentDidAppear() {// this.registerListeners();
  }
  /**
   * [ react-native-navigation method. ]
   *
   * Executed when the component is navigated away from view.
   */


  componentDidDisappear() {}

  componentWillUnmount() {
    this.removeListeners();
  }
  /**
   * Registers all the listenrs for this component
   */


  render() {
    /** Props */
    const {
      style,
      snapPoints
    } = this.props;
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(TapGestureHandler, {
      maxDurationMs: 100000,
      onHandlerStateChange: this.handleTapEvent
    }, /*#__PURE__*/React.createElement(View, {
      style: styles.containerStyle
    }, /*#__PURE__*/React.createElement(TouchableWithoutFeedback, {
      onPress: this.touchedOutside
    }, /*#__PURE__*/React.createElement(Animated.View, {
      style: [styles.overlayStyle, {
        opacity: this._masterOpacity
      }]
    })), /*#__PURE__*/React.createElement(Animated.View, {
      style: [{
        backgroundColor: this.props.backgroundColor,
        borderTopLeftRadius: this.props.borderRadius,
        borderTopRightRadius: this.props.borderRadius
      }, style, {
        height: this.topSnap,
        width: '100%',
        transform: [{
          translateY: this._masterTranslateY
        }]
      }]
    }, /*#__PURE__*/React.createElement(PanGestureHandler, {
      onGestureEvent: this._onGestureEvent,
      onHandlerStateChange: this._onGestureEvent
    }, /*#__PURE__*/React.createElement(Animated.View, {
      onLayout: this.handleLayoutHeader,
      style: {
        zIndex: 101
      }
    }, this.props.renderHeader && this.props.renderHeader())), /*#__PURE__*/React.createElement(Animated.View, {
      style: {
        overflow: 'hidden',
        width: '100%'
      }
    }, /*#__PURE__*/React.createElement(PanGestureHandler, {
      onGestureEvent: this._onGestureEventScrolling,
      onHandlerStateChange: this._onGestureEventScrolling
    }, /*#__PURE__*/React.createElement(Animated.View, {
      style: {
        height: this.topSnap
      }
    }, /*#__PURE__*/React.createElement(Animated.View, {
      onLayout: this.handleLayoutContent,
      style: [{
        transform: [{
          translateY: this._masterScrollY
        }]
      }]
    }, this.props.renderContent && this.props.renderContent()))))))));
  }

}

_defineProperty(BottomSheet, "defaultProps", {
  snapPoints: [0, 100, 300, 500],
  backgroundColor: '#FFF',
  dismissWhenTouchOutside: true,
  fadeOpacity: 0.7,
  borderRadius: 0,
  enabledContentGestureInteraction: true
});

export default gestureHandlerRootHOC(BottomSheet);
const styles = StyleSheet.create({
  containerStyle: {
    flex: 1,
    flexDirection: 'row'
  },
  overlayStyle: {
    position: 'absolute',
    width: '100%',
    height: '100%',
    backgroundColor: '#000'
  }
});
//# sourceMappingURL=BottomSheet.js.map